/*
 * Oscar - Open Service Container ARchitecture.
 * An implementation of the Open Services Gateway Initiative (OSGi).
 * Copyright (C) 2002  Richard S. Hall
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Contact: Richard S. Hall (heavy@ungoverned.org)
 * Contributor(s):
 *
**/
package org.ungoverned.osgi.bundle.httpadmin;

import javax.servlet.Servlet;

import org.osgi.framework.*;
import org.osgi.service.http.HttpContext;
import org.osgi.service.http.HttpService;
import org.ungoverned.osgi.bundle.httpadmin.util.SimpleContext;

/**
 * The bundle activator object is responsibe for starting and stoppig an OSGi bundle.
 * It should perform initialization and cleanup actions whenever bundle is started or
 * stopped. This Activator registers simple Http Servler that makes possible
 * to start/stop/update bundles using simple web interface.
**/
public class Activator implements BundleActivator, ServiceListener
{
    private static final String aliasRes = "/admin/resource";
    private static final String aliasServlet = "/admin";
    private Servlet servlet;
    private HttpContext context;
    private BundleContext bc;
    private ServiceReference ref;

    public void start(BundleContext bc) throws Exception
    {
        this.bc = bc;
        context = new SimpleContext(this);
        servlet = new HttpAdmin(bc);
        registerServlet();
        bc.addServiceListener(this,
            "(objectClass=" + HttpService.class.getName() +
            ")");
    }

    public void stop(BundleContext bc) throws Exception
    {
        try {
            unregisterServlet();
        } catch (Throwable t) {
            t.printStackTrace();
        }

        // clear reference to ease garbage collector ;)
        servlet = null;
        context = null;
        bc = null;
        ref = null;
    }

    // implement service listener
    public void serviceChanged(ServiceEvent event)
    {
        switch (event.getType())
        {
            case ServiceEvent.REGISTERED:
                registerServlet();
                break;

            case ServiceEvent.UNREGISTERING:
                unregisterServlet();
                break;
        }
    }

    private void registerServlet()
    {
        if (ref == null) //init ref
        {
            ref = bc.getServiceReference(HttpService.class.getName());
        }

        if (ref != null) //if got reference
        {
            try {
                HttpService http = (HttpService) bc.getService(ref);
                http.registerServlet(aliasServlet, servlet, null, context);
                http.registerResources(aliasRes,
                    "org/ungoverned/osgi/bundle/httpadmin",
                    null);
            } catch (Throwable t) {
                t.printStackTrace();
            }
        }
    }

    private void unregisterServlet()
    {
        if (ref != null)
        {
            HttpService http = (HttpService) bc.getService(ref);

            try {
                if (http != null)
                {
//                    http.unregister(aliasRes);
                    http.unregister(aliasServlet);
                }
            } catch (IllegalArgumentException iae) {
                // this should never happen!
                iae.printStackTrace();
            }

            bc.ungetService(ref);
            ref = null;
        }
    }
}
