/*
 * Oscar - Open Service Container ARchitecture.
 * An implementation of the Open Services Gateway Initiative (OSGi).
 * Copyright (C) 2002  Richard S. Hall
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Contact: Richard S. Hall (heavy@ungoverned.org)
 * Contributor(s):
 *
**/
package org.ungoverned.osgi.bundle.httpadmin;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Dictionary;
import java.util.Enumeration;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.ungoverned.osgi.bundle.httpadmin.util.Servlet;
import org.ungoverned.osgi.bundle.httpadmin.util.Text;

/**
 * This is the servlet which will serve users requiest. It created html ui.
 */
public class HttpAdmin extends HttpServlet
{
    private static String framesetHtml;
    private static String infoHtml;
    private static Servlet helper;
    private BundleContext bc;

    public HttpAdmin(BundleContext bc) throws Exception
    {
        this.bc = bc;
        framesetHtml = Text.readTextResource(this, "frameset.html");
        infoHtml = Text.readTextResource(this, "info.html");
        helper = new Servlet();
    }

    protected void doGet(HttpServletRequest request,  HttpServletResponse response)
        throws IOException
    {
        String action = request.getParameter("action");

        if (Text.isEmptyString(action)) //if action is not set then draw frameset
        {
            PrintWriter out = response.getWriter();
            out.write(framesetHtml);
            out.flush();
        }
        else if (action.equals("list"))
        {
            processList(response);
        }
        else if (action.equals("uninstall")) // This is handled in other way
        {
            PrintWriter out = response.getWriter();

            try {
                bc.getBundle(Long.parseLong(request.getParameter("bid"))).uninstall();
                helper.printInfoPage("Info",
                    "Please click on a bundle from the list on the left side",
                    out);
            } catch (Throwable t) {
                helper.printErrorPage(
                    "Unable to uninstall bundle.<p>Exception: <br>" + t,
                    out);
            }

            out.flush();
        }
        else
        {
            processInfo(request, response);
        }

        response.flushBuffer();
    }

    // processes action=list
    private void processList(HttpServletResponse response) throws IOException
    {
        PrintWriter out = response.getWriter();
        StringBuffer sb = new StringBuffer("<center>");
        Bundle[] bundles = bc.getBundles();

        for (int i = 1; i < bundles.length; i++) // dont' show syste bundle with ID=0
        {
            String name = null;

            try {
                name = (String) bundles[i].getHeaders().get("Bundle-Name");
            } catch (NullPointerException e) {
            }

            if (Text.isEmptyString(name))
            {
                name = "No-Name (" + bundles[i].getBundleId() + ")";
            }

            //sb.append("<br><a href=\"servlet?action=info&bid=").append(bundles[i].getBundleId())
            sb.append("<br><a href=\"admin?action=info&bid=").append(bundles[i].getBundleId())
                .append("\" target=right>").append(name).append("</a>");
        }

        //sb.append("<p><a href=\"servlet?action=list\"><b>Refresh</b></a></center></p>");
        sb.append("<p><a href=\"admin?action=list\"><b>Refresh</b></a></center></p>");
        helper.printInfoPage("Bundles List", sb.toString(), out);
    }

    // action=info,start,stop,update,
    private void processInfo(HttpServletRequest request,
        HttpServletResponse response)
        throws IOException
    {
        PrintWriter out = response.getWriter();
        String tmp = request.getParameter("bid");
        Bundle bundle;

        try {
            bundle = bc.getBundle(Long.parseLong(tmp));
        } catch (Throwable t) {
            bundle = null;
        }

        if (bundle == null)
        {
            if (tmp == null)
            {
                helper.printInfoPage("Info",
                    "Please click on a bundle from the list on the left side",
                    out);
            }
            else
            {
                helper.printErrorPage(
                    "Unable to find bundle with ID " + tmp +
                    "<BR>It's probably uninstalled.", out);
            }
        }
        else
        {
            // process action
            tmp = request.getParameter("action");

            try {
                if (tmp.equals("start"))
                {
                    bundle.start();
                }
                else if (tmp.equals("stop"))
                {
                    bundle.stop();
                }
                else if (tmp.equals("update"))
                {
                    bundle.update();
                }
            } catch (Throwable e) {
                helper.printErrorPage(
                    "Unable to " + tmp + " bundle<p>Exception:<br>" + e,
                    out);
                response.flushBuffer();
                return;
            }

            // build http ui
            try {
                StringBuffer sb = new StringBuffer("<BR><BR>").append(getButtonsHtml(
                    bundle)).append(getInfoHtml(bundle)).append(getHeadersHtml(
                    bundle)).append(getServicesHtml(bundle));
                out.write(Text.replaceString(infoHtml, "$INFO", sb.toString()));
            } catch (Throwable t) {
                helper.printErrorPage(
                    "Unable to complete bundle operation. Possibly due to unsifficient priviledges" +
                    "<p>Exception:<br>" + t, out);
            }
        }

        response.flushBuffer();
    }

    private static final String tableHeader = "<table class='INFO-TABLE' border=1 cellspacing=1 width=100%><caption class='INFO-TITLE'>";

    private String getButtonsHtml(Bundle bundle)
    {
        //String tmp = "<td class='INFO-TD'><a href=\"servlet?bid=" +
        String tmp = "<td class='INFO-TD'><a href=\"admin?bid=" +
            bundle.getBundleId();
        StringBuffer sb = new StringBuffer();
        sb.append(tableHeader).append(
            "<b>Bundle Controls</b></caption><tr class='INFO-TR'>").append(tmp).append(
            "&action=start\"><b>Start</b></a></td>").append(tmp).append(
            "&action=stop\"><b>Stop</b></a></td>").append(tmp).append(
            "&action=update\"><b>Update</b></a></td>").append(tmp).append(
            "&action=uninstall\"><b>Uninstall</b></a></td>").append(
            "</tr></table>");
        return sb.toString();
    }

    private String getInfoHtml(Bundle bundle)
    {
        StringBuffer sb = new StringBuffer();
        sb.append(tableHeader).append("<b>Bundle Information</b></caption>").append(
            "<tr class='INFO-TR'><td class='INFO-TD'>").append(
            "This bundle (").append(bundle.getLocation()).append(
            ") is installed with ID = ").append(bundle.getBundleId()).append(
            "<BR>");

        switch (bundle.getState())
        {
            case Bundle.INSTALLED:
                sb.append("And is recently INSTALLED");
                break;

            case Bundle.RESOLVED:
                sb.append("And is RESOLVED");
                break;

            case Bundle.STARTING:
                sb.append("And is STARTING");
                break;

            case Bundle.STOPPING:
                sb.append("And is STOPPING");
                break;

            case Bundle.ACTIVE:
                sb.append("And is ACTIVE");
                break;

            default:
                sb.append("Unfortunately it's state cannot be determined.");
        }

        sb.append("</td></tr></table>");

        return sb.toString();
    }

    private String getHeadersHtml(Bundle bundle)
    {
        Dictionary props = bundle.getHeaders();
        StringBuffer sb = new StringBuffer();
        sb.append(tableHeader).append("<b>Manifest Headers</b></caption>");

        for (Enumeration e = props.keys(); e.hasMoreElements();)
        {
            Object key = e.nextElement();
            sb.append("<tr class='INFO-TR'><td class='INFO-TD' nowrap>").append(
                key).append("</td><td class='INFO-TD'>").append(props.get(key))
                .append("</td></tr>");
        }

        sb.append("</table>");

        return sb.toString();
    }

    private String getServicesHtml(Bundle bundle)
    {
        StringBuffer sb = new StringBuffer();
        ServiceReference[] exported = bundle.getRegisteredServices();
        ServiceReference[] imported = bundle.getServicesInUse();
        sb.append(tableHeader).append("<b>Services</b></caption>");

        if (exported != null)
        {
            sb.append("<tr class='INFO-TR'><td class='INFO-TD' nowrap>Registered Services</td><td class='INFO-TD'>");

            for (int i = 0; i < exported.length; i++)
            {
                String[] names = (String[]) exported[i].getProperty(
                    "objectClass");

                for (int j = 0; j < names.length; j++)
                {
                    sb.append(names[j]).append("<br>");
                }
            }

            sb.append("</td></tr>");
        }

        if (imported != null)
        {
            sb.append("<tr class='INFO-TR'><td class='INFO-TD'>Used Services</td><td class='INFO-TD'>");

            for (int i = 0; i < imported.length; i++)
            {
                String[] names = (String[]) imported[i].getProperty(
                    "objectClass");

                for (int j = 0; j < names.length; j++)
                {
                    sb.append(names[j]).append("<br>");
                }
            }

            sb.append("</td></tr>");
        }

        if (imported == null && exported == null)
        {
            sb.append("<tr class='INFO-TR'><td class='INFO-TD'>Neither used or registered</td></tr>");
        }

        sb.append("</table><br>");

        return sb.toString();
    }
}