/*
 * Oscar - Open Service Container ARchitecture.
 * An implementation of the Open Services Gateway Initiative (OSGi).
 * Copyright (C) 2002  Richard S. Hall
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Contact: Richard S. Hall (heavy@ungoverned.org)
 * Contributor(s):
 *
**/
package org.ungoverned.osgi.bundle.httpadmin.util;

import java.net.URL;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.osgi.service.http.HttpContext;

/**
 * This is just a helper - default HttpContext implementation for an OSGi HTTP Service.
 * It does not handle security and by default permits ANY access. It alse returns
 * null as mime type to indicate that default mime types should be used.
**/
public class SimpleContext implements HttpContext
{
    private Object base;

    /**
     * Constructs a new simple http context to use with OSGi HTTP Service.
     * @param base a reference to the object which is used as base for resources
     *  which are considered to be relative to that object package name (see getResource).
    **/
    public SimpleContext(Object base)
    {
        this.base = base;
    }

    /**
     * This method is resposible for security - e.g http sessions and logins.
     * Actually default implementation will always allow ANY acces.
     *
     * @return always true - e.g. access is allowed.
    **/
    public boolean handleSecurity(HttpServletRequest request,
        HttpServletResponse response)
    {
        return true;
    }

    /**
     * Returns <code>null</code> which is considered to be default mime type
    **/
    public String getMimeType(String resource)
    {
        return null;
    }

    /**
     * Reads a resource as URL. These resources should be relative to the base
     * object which was given when SimpleContext class was instantiated.
    **/
    public URL getResource(String name)
    {
        return base.getClass().getResource(name);
    }
}