/*
 * Oscar - Open Service Container ARchitecture.
 * An implementation of the Open Services Gateway Initiative (OSGi).
 * Copyright (C) 2002  Richard S. Hall
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Contact: Richard S. Hall (heavy@ungoverned.org)
 * Contributor(s):
 *
**/
package org.ungoverned.osgi.bundle.httpadmin.util;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * A simple utility class - helper for strings
**/
public class Text
{
    /**
     * Creates hex string from <code>byte</code> value. The result string is 2 chars long String.
     *
     * @param value byte to convert to hex string
     * @return hex string
    **/
    public static String hexString(byte value)
    {
        return hexString(value, 2);
    }

    /**
     * Creates hex string from <code>int</code> value. The result string is 4 chars long String.
     *
     * @param value int to convert to hex string
     * @return hex string
    **/
    public static String hexString(int value)
    {
        return hexString(value, 4);
    }

    /**
     * Hex string with the given mininum length. If needed, '0'(s) are add in
     * front.
     *
     * @param value int to convert to hex string
     * @param length the minimum length of returned string.
     * @return hex string
    **/
    public static String hexString(int value, int length)
    {
        StringBuffer result = new StringBuffer();
        result.append(Integer.toHexString(value).toUpperCase());

        while (result.length() < length)
        {
            result.insert(0, "0");
        }

        return result.append("h").toString();
    }

    /**
     * This static method is simple search and replace engine. If looks for a certain
     * string in source string and replaces it with another.
     *
     * @param source is the source string
     * @param find is the search string to be replaced.
     * @param replace is replaces string.
    **/
    public static String replaceString(String source, String find,
        String replace)
    {
        StringBuffer result = new StringBuffer("");
        int maxlength = source.length();
        int start = 0;
        int findlength = find.length();
        int lastIndex;
        int i = source.indexOf(find);

        if (i < 0)
        {
            lastIndex = maxlength;
            result = new StringBuffer(source);
        }
        else
        {
            lastIndex = i + findlength;
        }

        while (lastIndex < maxlength)
        {
            result.append(source.substring(start, i));
            result.append(replace);
            i = source.indexOf(find, lastIndex);
            start = lastIndex;

            if (i < 0)
            {
                result.append(source.substring(start, maxlength));
                break;
            }

            lastIndex = i + findlength;
        }

        return result.toString();
    }

    /**
     * Simple file reader with non-blocking io. This can be used without
     * any troubles with jdk 1.1. It reads file bytes from (jar) file as resource.
     *
     * NOTE: do not use this method with non-text files.
     *
     * @param obj ANY object that can be used for base. template name is
     *           relative to that object. (see getResourceAsStream)
     * @param file name of resource file.
     *
     * @return byte[] contents of resource
    **/
    public static byte[] readResource(Object obj, String file)
    {
        InputStream is = obj.getClass().getResourceAsStream(file);

        if (is == null)
        {
            return null;
        }

        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        byte[] buf = new byte[1024];

        try {
            int i;

            do
            {
                i = is.read(buf);
                if (i != -1)
                    baos.write(buf, 0, i);
            }
            while (i != -1);

            is.close();
        } catch (IOException e) {
            try {
                is.close();
            } catch (IOException ex) {
            }

            return null;
        }

        return (baos.toByteArray());
    }

    /**
     * Simple file reader with non-blocking io. This can be used without
     * any troubles with jdk 1.1. It reads file bytes from (jar) file as resource.
     *
     * NOTE: do not use this method with non-text files.
     *
     * @param obj ANY object that can be used for base. template name is
     *           relative to that object. (see getResourceAsStream)
     * @param file name of resource file.
     *
     * @return String with contents of file.
    **/
    public static String readTextResource(Object obj, String file)
    {
        return new String(readResource(obj, file));
    }

    /**
     * Checks if string is empty - either null or ""
     * @return true if the string is null or ""
    **/
    public static boolean isEmptyString(String s)
    {
        return (s == null) || (s.length() == 0);
    }
}